package edu.uoc.petfinder.configuration;

import com.google.common.base.Predicate;
import com.sun.org.apache.xpath.internal.operations.Bool;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.context.annotation.Configuration;
import org.springframework.http.ResponseEntity;
import springfox.documentation.RequestHandler;
import springfox.documentation.builders.ApiInfoBuilder;
import springfox.documentation.builders.PathSelectors;
import springfox.documentation.builders.RequestHandlerSelectors;
import springfox.documentation.service.ApiInfo;
import springfox.documentation.spi.DocumentationType;
import springfox.documentation.spring.web.plugins.Docket;
import springfox.documentation.swagger.web.*;
import springfox.documentation.swagger2.annotations.EnableSwagger2;

import java.time.ZonedDateTime;

@Configuration
@ComponentScan(basePackages = {"edu.uoc.petfinder"})
@EnableSwagger2
public class SwaggerConfiguration {


    @Bean
    public Docket eDesignApi() {
        return new Docket(DocumentationType.SWAGGER_2)
                    .apiInfo(apiInfo())
                    .enable(Boolean.TRUE).select().apis(RequestHandlerSelectors.any())
                    .paths(PathSelectors.any())
                    .apis(RequestHandlerSelectors.basePackage("edu.uoc.petfinder.controller"))
                .build()
                .pathMapping("/").directModelSubstitute(ZonedDateTime.class, String.class)
                .genericModelSubstitutes(ResponseEntity.class)
                .useDefaultResponseMessages(Boolean.TRUE)
                .enableUrlTemplating(Boolean.TRUE);
    }
    @Bean
    public UiConfiguration uiConfig() {
        return UiConfigurationBuilder.builder()
                .deepLinking(Boolean.TRUE)
                .displayOperationId(Boolean.TRUE)
                .defaultModelRendering(ModelRendering.EXAMPLE)
                .docExpansion(DocExpansion.NONE)
                .filter(Boolean.TRUE).maxDisplayedTags(100).operationsSorter(OperationsSorter.ALPHA)
                .showExtensions(Boolean.TRUE).tagsSorter(TagsSorter.ALPHA)
                .supportedSubmitMethods(UiConfiguration.Constants.DEFAULT_SUBMIT_METHODS)
                .validatorUrl(null)
                .build();
    }
    private ApiInfo apiInfo() {
        return new ApiInfoBuilder().title("PetFinder API")
                .description("Swagger Doc for PetFinder Backend API")
                .version("1.0.0-SNAPSHOT")
                .build();
    }
}
