package edu.uoc.petfinder.controller;

import edu.uoc.petfinder.dto.ActionDto;
import edu.uoc.petfinder.dto.PetDto;
import edu.uoc.petfinder.dto.UserDto;
import edu.uoc.petfinder.service.ActionService;
import edu.uoc.petfinder.service.PetService;
import lombok.AllArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;

@RestController
@RequestMapping("/pet")
@AllArgsConstructor
public class ActionController {

    private ActionService actionService;

    private PetService petService;

    @PreAuthorize("isAuthenticated()")
    @PostMapping("/{petId}/lost")
    public ResponseEntity lost(@PathVariable String petId, @Valid @RequestBody ActionDto actionDto, @RequestAttribute(name = "user") UserDto userDto) {
        PetDto petDto = this.petService.getPetById(petId);
        petDto = this.actionService.lost(actionDto, petDto, userDto);
        petDto = this.petService.updatePet(petDto, userDto.getEmail());
        return ResponseEntity.status(HttpStatus.CREATED).body(petDto);
    }

    @PreAuthorize("isAuthenticated()")
    @PostMapping("/{petId}/found")
    public ResponseEntity found(@PathVariable String petId, @Valid @RequestBody ActionDto actionDto, @RequestAttribute(name = "user") UserDto userDto) {
        PetDto petDto = this.petService.getPetById(petId);
        petDto = this.actionService.found(actionDto, petDto, userDto);
        petDto = this.petService.updatePet(petDto, userDto.getEmail());
        return ResponseEntity.status(HttpStatus.CREATED).body(petDto);
    }

    @PreAuthorize("isAuthenticated()")
    @PostMapping("/{petId}/sighting")
    public ResponseEntity sighting(@PathVariable String petId, @Valid @RequestBody ActionDto actionDto, @RequestAttribute(name = "user") UserDto userDto) {
        PetDto petDto = this.petService.getPetById(petId);
        petDto = this.actionService.sighting(actionDto, petDto, userDto);
        petDto = this.petService.updatePet(petDto, userDto.getEmail());
        return ResponseEntity.status(HttpStatus.CREATED).body(petDto);
    }
}
