package edu.uoc.petfinder.controller;

import edu.uoc.petfinder.dto.SimplePetDto;
import edu.uoc.petfinder.dto.UserDto;
import edu.uoc.petfinder.service.PetService;
import edu.uoc.petfinder.service.UserService;
import lombok.RequiredArgsConstructor;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

@RestController
@RequestMapping("/admin")
@PreAuthorize("hasRole('ROLE_ADMIN')")
@RequiredArgsConstructor
public class AdminController {

    private final UserService userService;

    private final PetService petService;

    @GetMapping("/users")
    public ResponseEntity fetchUsers(@RequestParam(required = false) String search, Pageable pageable) {
        Page<UserDto> userPage = this.userService.searchUsers(search, pageable);
        return ResponseEntity.ok(userPage);
    }

    @GetMapping("/pets")
    public ResponseEntity fetchPets(@RequestParam(required = false) String search, Pageable pageable) {
        Page<SimplePetDto> petsPage = this.petService.searchPets(search, pageable);
        return ResponseEntity.ok(petsPage);
    }

    @DeleteMapping("/pet/{id}")
    public ResponseEntity deletePet(@PathVariable("id") String petId, @RequestAttribute(name = "user") UserDto userDto ){
        this.petService.deletePet(petId, Boolean.FALSE, userDto);
        return ResponseEntity.noContent().build();
    }

    @DeleteMapping("/user/{id}")
    public ResponseEntity deleteUser(@PathVariable("id") String userId){
        this.userService.deleteUser(userId);
        return ResponseEntity.noContent().build();
    }
}
