package edu.uoc.petfinder.controller;

import edu.uoc.petfinder.dto.LoginResponseDto;
import edu.uoc.petfinder.dto.SignInUserDto;
import edu.uoc.petfinder.dto.TokenDto;
import edu.uoc.petfinder.dto.UserDto;
import edu.uoc.petfinder.security.AuthenticationService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import lombok.NoArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;

@Api(description = "Authorization Controller to handle auth request, social login and users signup")
@RestController
@RequestMapping("/auth")
@AllArgsConstructor
public class AuthController {

    private AuthenticationService authenticationService;

    @ApiOperation(value = "User login endpoint")
    @PostMapping("/signin")
    public ResponseEntity userSignIn(@Valid @RequestBody SignInUserDto signInUserDto) {
        UserDto userDto = UserDto.builder().email(signInUserDto.getEmail()).password(signInUserDto.getPassword()).build();
        LoginResponseDto responseDto = this.authenticationService.signIn(userDto);
        return ResponseEntity.ok(responseDto);
    }

    @ApiOperation(value = "User register endpoint")
    @PostMapping("/signup")
    public ResponseEntity userSignUp(@Valid @RequestBody UserDto userDto) {
        LoginResponseDto responseDto = this.authenticationService.signUp(userDto);
        return ResponseEntity.status(HttpStatus.CREATED).body(responseDto);
    }

    @ApiOperation(value = "Social Facebook endpoint for register and login users")
    @PostMapping("/facebook")
    public ResponseEntity facebookSignIn(@RequestBody TokenDto tokenDto) {
        LoginResponseDto responseDto = this.authenticationService.connectWithFacebook(tokenDto);
        return ResponseEntity.ok(responseDto);
    }

    @ApiOperation(value = "Social Google endpoint for register and login users")
    @PostMapping("/google")
    public ResponseEntity googleSignIn(@RequestBody TokenDto tokenDto) {
        LoginResponseDto responseDto = this.authenticationService.connectWithGoogle(tokenDto);
        return ResponseEntity.ok(responseDto);
    }
}
