package edu.uoc.petfinder.controller;

import edu.uoc.petfinder.dto.SimplePetDto;
import edu.uoc.petfinder.dto.UserDto;
import edu.uoc.petfinder.exception.PetFinderException;
import edu.uoc.petfinder.mail.EmailService;
import edu.uoc.petfinder.service.UserService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.util.Collections;

@Api(description = "User Controller to handle user operations")
@RestController
@RequestMapping("/user")
@AllArgsConstructor
public class UserController {

    private UserService userService;

    private EmailService emailService;


    @ApiOperation(value = "Method to fetch complet info of the own user")
    @GetMapping
    @PreAuthorize("isAuthenticated()")
    public ResponseEntity getOwnUserById(@RequestParam String email, @RequestAttribute(name = "user") UserDto userDto) {
        if (email.equalsIgnoreCase(userDto.getEmail())) {
           return ResponseEntity.ok(userDto);
        }
        throw new PetFinderException("Error: User not found", HttpStatus.NOT_FOUND);
    }

    @ApiOperation(value = "Method to updated many fields of own user")
    @PutMapping
    @PreAuthorize("isAuthenticated()")
    public ResponseEntity updateOwnUser(@RequestParam String email, @RequestAttribute(name = "user") UserDto connectedUser, @RequestBody UserDto updatedUserDto) {
        if (!email.equalsIgnoreCase(connectedUser.getEmail())) {
            throw new PetFinderException("Error: User not found", HttpStatus.NOT_FOUND);
        }
        updatedUserDto = this.userService.patchUserFields(connectedUser, updatedUserDto);
        return ResponseEntity.ok(updatedUserDto);
    };

    @ApiOperation(value = "Method to verify if an email available for news users")
    @GetMapping(value = "/available")
    public ResponseEntity isEmailAvailable(@RequestParam String email) {
        Boolean isAvailable = this.userService.isEmailAvailable(email);
        return ResponseEntity.ok(Collections.singletonMap("isEmailAvailable", isAvailable));
    }

    @ApiOperation(value = "Method to fetch paginated pets for own user")
    @GetMapping(value = "/pet")
    public ResponseEntity fetchUserPets(Pageable pageable, @RequestAttribute(name = "user") UserDto connectedUser) {
        Page<SimplePetDto> pets = this.userService.fetchPaginatedPetsByUser(connectedUser, pageable);
        return ResponseEntity.ok(pets);
    }


}
