package edu.uoc.petfinder.dto;

import com.mongodb.client.model.geojson.Point;
import edu.uoc.petfinder.constant.Genre;
import edu.uoc.petfinder.constant.PetType;
import edu.uoc.petfinder.model.Action;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.springframework.data.mongodb.core.mapping.DBRef;
import org.springframework.util.CollectionUtils;

import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;
import java.time.Instant;
import java.time.ZonedDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

@Data
@Builder
@AllArgsConstructor
@NoArgsConstructor
public class PetDto {

    private String id;

    @NotNull
    private PetType type;

    @NotNull
    private Genre genre;

    @NotEmpty
    private String name;

    private ZonedDateTime birthDate;

    @Size(max = 150)
    private String description;

    @NotNull
    private LocationDto location;

    private UserDto owner;

    private Instant createdDate;

    private Instant lastModifiedDate;

    private ActionDto lastState;

    private List<ActionDto> actions = new ArrayList<>();

    private List<ImageDto> images = new ArrayList<>();

    public void addImage(ImageDto imageDto) {
        if (images == null) {
            images = new ArrayList<>();
        }
        images.add(imageDto);
    }

    public void removeImage(ImageDto imageDto) {
        if (!CollectionUtils.isEmpty(images)) {
            images = images.stream().filter(imag -> !imag.equals(imageDto)).collect(Collectors.toList());
        }
    }

    public void addAction(ActionDto actionDto) {
        if (actions == null) {
            actions = new ArrayList<>();
        }
        actions.add(actionDto);
        setLastState(actionDto);
    }
}
