package edu.uoc.petfinder.exception;

import org.springframework.core.Ordered;
import org.springframework.core.annotation.Order;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.http.converter.HttpMessageNotReadableException;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.context.request.WebRequest;
import org.springframework.web.servlet.mvc.method.annotation.ResponseEntityExceptionHandler;

import java.time.ZonedDateTime;
import java.util.List;
import java.util.stream.Collectors;

@ControllerAdvice
@Order(Ordered.HIGHEST_PRECEDENCE)
public class ApiExceptionHandler extends ResponseEntityExceptionHandler {

    @Override
    protected ResponseEntity<Object> handleHttpMessageNotReadable(HttpMessageNotReadableException ex, HttpHeaders headers, HttpStatus status, WebRequest request) {
        ErrorResponse errorResponse = ErrorResponse.builder()
                .message("Unreadable JSON")
                .status(HttpStatus.BAD_REQUEST)
                .debugMessage(ex.getLocalizedMessage())
                .timestamp(ZonedDateTime.now())
                .build();
        return ResponseEntity.badRequest().body(errorResponse);
    }

    @Override
    protected ResponseEntity<Object> handleMethodArgumentNotValid(MethodArgumentNotValidException ex, HttpHeaders headers, HttpStatus status, WebRequest request) {
        List<SubError> errors = ex.getBindingResult().getFieldErrors().stream()
                .map(er -> (ValidationSubError.builder().field(er.getField()).message(er.getDefaultMessage()).build()))
                .collect(Collectors.toList());
        ErrorResponse errorResponse = ErrorResponse.builder()
                .message("Body Request not valid")
                .status(HttpStatus.BAD_REQUEST)
                .debugMessage(ex.getLocalizedMessage())
                .timestamp(ZonedDateTime.now())
                .subErrors(errors)
                .build();
        return ResponseEntity.badRequest().body(errorResponse);
    }

    @ExceptionHandler(PetFinderException.class)
    public ResponseEntity handlePetFinderException(PetFinderException ex) {
        ErrorResponse errorResponse = ErrorResponse.builder()
                .message(ex.getMessage())
                .status(ex.getStatus())
                .timestamp(ZonedDateTime.now())
                .code(ex.getCode())
                .build();
        return ResponseEntity.status(errorResponse.getStatus()).body(errorResponse);
    }
}
