package edu.uoc.petfinder.mail;

import edu.uoc.petfinder.constant.ActionType;
import edu.uoc.petfinder.dto.LocationDto;
import edu.uoc.petfinder.spatial.impl.SpatialOperations;
import lombok.RequiredArgsConstructor;
import org.springframework.data.geo.Distance;
import org.springframework.stereotype.Component;

import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import java.util.HashMap;

@Component
@RequiredArgsConstructor
public class EmailFactory {

    private final EmailService emailService;

    private final SpatialOperations spatialOperations;

    public void accountCreatedMail(@NotNull EmailData emailData) {
        HashMap map = new HashMap<String, Object>();
        map.put("data", emailData);
        this.emailService.sendTemplatedMessage(emailData.getUser().getEmail(), "Account created on PetFinder", "account-created", map);
    }

    public void sightingMail(@NotNull EmailData emailData) {
        LocationDto lostLocation = this.extractLostLocation(emailData);
        LocationDto sightingLocation = emailData.getPet().getLastState().getLocation();
        Distance distance = this.spatialOperations.calculateDistanceBetweenTwoLocations(lostLocation, sightingLocation);
        emailData.setDistance(distance);
        HashMap map = new HashMap<String, Object>();
        map.put("data", emailData);
        this.emailService.sendTemplatedMessage(emailData.getPet().getOwner().getEmail(), String.format("Update: %s has been seen.", emailData.getPet().getName()), "pet-sighting", map);
    }

    public void foundMail(@NotNull EmailData emailData) {
        LocationDto lostLocation = this.extractLostLocation(emailData);
        LocationDto foundLocation = emailData.getPet().getLastState().getLocation();
        Distance distance = this.spatialOperations.calculateDistanceBetweenTwoLocations(lostLocation, foundLocation);
        emailData.setDistance(distance);
        HashMap map = new HashMap<String, Object>();
        map.put("data", emailData);
        this.emailService.sendTemplatedMessage(emailData.getPet().getOwner().getEmail(), String.format("We found %s !", emailData.getPet().getName()), "pet-found", map);
    }

    private LocationDto extractLostLocation(@NotNull EmailData emailData) {
        return emailData.getPet().getActions().stream()
                .filter(actionDto -> actionDto.getType().equals(ActionType.LOST))
                .map(actionDto -> actionDto.getLocation())
                .findFirst().get();
    }

}
