package edu.uoc.petfinder.mail.impl;

import edu.uoc.petfinder.mail.EmailService;
import lombok.AllArgsConstructor;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.io.Resource;
import org.springframework.mail.SimpleMailMessage;
import org.springframework.mail.javamail.JavaMailSender;
import org.springframework.mail.javamail.MimeMessageHelper;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.thymeleaf.context.Context;
import org.thymeleaf.spring5.SpringTemplateEngine;

import javax.mail.MessagingException;
import javax.mail.internet.MimeMessage;
import java.util.Map;

@Service
@RequiredArgsConstructor
@Slf4j
public class EmailServiceImpl implements EmailService {

    private final JavaMailSender emailSender;

    private final SpringTemplateEngine thymeleafTemplateEngine;

    @Value("${spring.mail.username}")
    private String from;

    @Value("${email.refer.base.link}")
    private String websiteBaseUrl;

    @Value("classpath:/static/images/logoShort.png")
    private Resource logoShortResource;

    @Value("classpath:/static/images/facebooklogo.png")
    private Resource facebookLogoResource;

    @Value("classpath:/static/images/twitterlogo.png")
    private Resource twitterLogoResource;

    @Value("classpath:/static/images/instagramlogo.png")
    private Resource instagramLogoResource;

    @Override
    @Deprecated
    @Async("emailPoolTaskExecutor")
    public void sendSimpleMessage(String to, String subject, String text) {
        SimpleMailMessage message = new SimpleMailMessage();
        message.setFrom(this.from);
        message.setTo(to);
        message.setSubject(subject);
        message.setText(text);
        emailSender.send(message);
    }

    @Override
    @Async("emailPoolTaskExecutor")
    public void sendTemplatedMessage(String to, String subject, String templateName, Map<String, Object> templateModel)
             {
        Context thymeleafContext = new Context();
        templateModel.put("websiteBaseUrl", this.websiteBaseUrl);
        thymeleafContext.setVariables(templateModel);
        String htmlBody = thymeleafTemplateEngine.process(StringUtils.appendIfMissing(templateName, ".html"), thymeleafContext);
         try {
             this.sendHtmlMessage(to, subject, htmlBody);
         } catch (MessagingException e) {
             log.error("Error trying to send mail: "+templateName, e);
         }
     }

    private void sendHtmlMessage(String to, String subject, String htmlBody) throws MessagingException {
        MimeMessage message = emailSender.createMimeMessage();
        MimeMessageHelper helper = new MimeMessageHelper(message, true, "UTF-8");
        helper.setTo(to);
        helper.setSubject(subject);
        helper.setText(htmlBody, true);
        helper.addInline("logo-short.png", this.logoShortResource);
        helper.addInline("facebook.png", this.facebookLogoResource);
        helper.addInline("twitter.png", this.twitterLogoResource);
        helper.addInline("instagram.png", this.instagramLogoResource);
        emailSender.send(message);
    }


}
