package edu.uoc.petfinder.model;

import com.mongodb.client.model.geojson.Point;
import edu.uoc.petfinder.constant.Genre;
import edu.uoc.petfinder.constant.PetType;
import edu.uoc.petfinder.dto.Location;
import lombok.Data;
import org.springframework.data.annotation.CreatedDate;
import org.springframework.data.annotation.Id;
import org.springframework.data.annotation.LastModifiedDate;
import org.springframework.data.mongodb.core.index.GeoSpatialIndexType;
import org.springframework.data.mongodb.core.index.GeoSpatialIndexed;
import org.springframework.data.mongodb.core.index.Indexed;
import org.springframework.data.mongodb.core.mapping.DBRef;
import org.springframework.data.mongodb.core.mapping.Document;

import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import java.time.Instant;
import java.time.ZonedDateTime;
import java.util.ArrayList;
import java.util.List;

@Document(collection = "PETS")
@Data
public class Pet {

    @Id
    private String id;

    private PetType type;

    private Genre genre;

    @NotEmpty
    @Indexed
    private String name;

    private ZonedDateTime birthDate;

    private String description;

    @GeoSpatialIndexed(type = GeoSpatialIndexType.GEO_2DSPHERE, name = "machant")
    private Location location;

    @NotNull
    @DBRef
    private User owner;

    private Action lastState;

    @DBRef
    private List<Action> actions = new ArrayList<>();

    @DBRef
    private List<Image> images = new ArrayList<>();

    @CreatedDate
    private Instant createdDate;

    @LastModifiedDate
    private Instant lastModifiedDate;

}
