package edu.uoc.petfinder.repository;

import edu.uoc.petfinder.dto.PetDto;
import edu.uoc.petfinder.dto.SimplePetDto;
import edu.uoc.petfinder.model.Pet;
import edu.uoc.petfinder.model.User;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;

import org.springframework.data.mongodb.repository.MongoRepository;
import org.springframework.data.mongodb.repository.Query;
import org.springframework.stereotype.Repository;

import java.util.Optional;
import java.util.stream.Stream;

@Repository
public interface PetRepository extends MongoRepository<Pet, String> {

    Optional<Pet> findByIdAndOwner(String id, User owner);

    Stream<Pet> findAllByOwner(User owner);

    @Query(value = "{ 'lastState.type' : { $ne : 'FOUND'}}")
    Page<Pet> findPaginatedOrderByDate(Pageable pageable);

    @Query(value = "{ 'lastState.type' : 'FOUND' }")
    Page<Pet> findFoundPetsPaginatedOrderByDate(Pageable pageable);

    @Query(value = "{ 'owner.id' : ?0 }", sort = "{ 'lastModifiedDate': -1 }")
    Page<SimplePetDto> findPaginatedPetsByUserId(String id, Pageable pageable);

}
