package edu.uoc.petfinder.security;

import edu.uoc.petfinder.exception.PetFinderException;
import io.jsonwebtoken.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpStatus;
import org.springframework.security.core.Authentication;
import org.springframework.stereotype.Service;

import java.util.Date;

@Service
@Slf4j
public class JwtTokenService {

    @Value("${petfinder.jwt.secret}")
    private String secret;

    @Value("${petfinder.jwt.expirationMillis}")
    private Integer expirationMillis;

    public String generateToken(Authentication authentication) {

        UserDetailsImpl userPrincipal = (UserDetailsImpl) authentication.getPrincipal();

        return Jwts.builder()
                .setSubject((userPrincipal.getUsername()))
                .setIssuedAt(new Date())
                .setExpiration(new Date((new Date()).getTime() + this.expirationMillis))
                .signWith(SignatureAlgorithm.HS512, this.secret)
                .compact();
    }

    public String getEmailFromToken(String token) {
        return Jwts.parser().setSigningKey(this.secret).parseClaimsJws(token).getBody().getSubject();
    }

    public boolean validateToken(String authToken) {
        try {
            Jwts.parser().setSigningKey(this.secret).parseClaimsJws(authToken);
            return true;
        } catch (SignatureException e) {
            log.error("Invalid JWT signature: {}", e.getMessage());
            throw new PetFinderException("Error: Invalid JWT signature", HttpStatus.UNAUTHORIZED );
        } catch (MalformedJwtException e) {
            log.error("Invalid JWT token: {}", e.getMessage());
            throw new PetFinderException("Error: Invalid JWT token", HttpStatus.UNAUTHORIZED );
        } catch (ExpiredJwtException e) {
            log.error("JWT token is expired: {}", e.getMessage());
            throw new PetFinderException("Error: JWT token is expired", HttpStatus.UNAUTHORIZED );
        } catch (UnsupportedJwtException e) {
            log.error("JWT token is unsupported: {}", e.getMessage());
            throw new PetFinderException("Error: JWT token is unsupported", HttpStatus.UNAUTHORIZED );
        } catch (IllegalArgumentException e) {
            log.error("JWT claims string is empty: {}", e.getMessage());
            throw new PetFinderException("Error: JWT claims string is empty", HttpStatus.UNAUTHORIZED );
        }
    }
}
