package edu.uoc.petfinder.service.impl;

import edu.uoc.petfinder.constant.ActionType;
import edu.uoc.petfinder.dto.ActionDto;
import edu.uoc.petfinder.dto.PetDto;
import edu.uoc.petfinder.dto.UserDto;
import edu.uoc.petfinder.exception.PetFinderException;
import edu.uoc.petfinder.mail.EmailData;
import edu.uoc.petfinder.mail.EmailFactory;
import edu.uoc.petfinder.mapper.ApiMapper;
import edu.uoc.petfinder.model.Action;
import edu.uoc.petfinder.repository.ActionRepository;
import edu.uoc.petfinder.service.ActionService;
import edu.uoc.petfinder.service.PetService;
import edu.uoc.petfinder.state.ActionContext;
import edu.uoc.petfinder.state.State;
import edu.uoc.petfinder.state.StateFactory;
import lombok.AllArgsConstructor;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;

@Service
@AllArgsConstructor
public class ActionServiceImpl implements ActionService {

    private ActionRepository actionRepository;

    private ApiMapper mapper;

    private ActionContext actionContext;

    private StateFactory stateFactory;

    private EmailFactory emailFactory;

    @Override
    public ActionDto saveAction(ActionDto actionDto) {
        Action savedAction = this.actionRepository.save(this.mapper.actionDtoToAction(actionDto));
        return this.mapper.actionToActionDto(savedAction);
    }

    @Override
    public PetDto lost(ActionDto actionDto, PetDto petDto, UserDto userDto) {
        if (!actionDto.getType().equals(ActionType.LOST)) {
            throw new PetFinderException("Error: Incorrect action type", HttpStatus.BAD_REQUEST);
        }
        this.prepareActionContext(actionDto, petDto, userDto);
        return this.actionContext.getState().lost(this.actionContext);
    }

    @Override
    public PetDto found(ActionDto actionDto, PetDto petDto, UserDto userDto) {
        if (!actionDto.getType().equals(ActionType.FOUND)) {
            throw new PetFinderException("Error: Incorrect action type", HttpStatus.BAD_REQUEST);
        }
        this.prepareActionContext(actionDto, petDto, userDto);
        petDto = this.actionContext.getState().found(this.actionContext);
        this.emailFactory.foundMail(EmailData.builder().user(userDto).action(actionDto).pet(petDto).build());
        return petDto;
    }

    @Override
    public PetDto sighting(ActionDto actionDto, PetDto petDto, UserDto userDto) {
        if (!actionDto.getType().equals(ActionType.SIGHTING)) {
            throw new PetFinderException("Error: Incorrect action type", HttpStatus.BAD_REQUEST);
        }
        this.prepareActionContext(actionDto, petDto, userDto);
        petDto = this.actionContext.getState().sighting(this.actionContext);
        this.emailFactory.sightingMail(EmailData.builder().user(userDto).action(actionDto).pet(petDto).build());
        return petDto;
    }

    private void prepareActionContext(ActionDto actionDto, PetDto petDto, UserDto userDto) {
        State state = this.stateFactory.getState(actionDto);
        this.actionContext.setActionDto(actionDto);
        this.actionContext.setPetDto(petDto);
        this.actionContext.setState(state);
        this.actionContext.setUserDto(userDto);
    }
}
