package edu.uoc.petfinder.service.impl;

import edu.uoc.petfinder.dto.ImageDto;
import edu.uoc.petfinder.service.ImageCompressor;
import org.apache.commons.io.FileUtils;
import org.apache.commons.io.FilenameUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.unit.DataSize;

import javax.imageio.IIOImage;
import javax.imageio.ImageIO;
import javax.imageio.ImageWriteParam;
import javax.imageio.ImageWriter;
import javax.imageio.stream.ImageInputStream;
import java.awt.image.BufferedImage;
import java.io.*;

@Service
public class ImageCompressorImpl implements ImageCompressor {

    @Override
    public InputStream compressImage(ImageDto imageDto, Float compressionRatio, String minCompressingFileSize) throws IOException {

        InputStream tempIS = this.checkFileSize(imageDto, minCompressingFileSize);
        if (tempIS != null) return tempIS;

        InputStream input = imageDto.getFileResource().getInputStream();
        BufferedImage image = ImageIO.read(input);

        ByteArrayOutputStream os = new ByteArrayOutputStream();
        String imageFormatString = FilenameUtils.getExtension(imageDto.getOriginalFileName());
        ImageWriter writer =  ImageIO.getImageWritersByFormatName(imageFormatString).next();

        ImageInputStream ios = ImageIO.createImageOutputStream(os);
        writer.setOutput(ios);

        ImageWriteParam param = writer.getDefaultWriteParam();

        param.setCompressionMode(ImageWriteParam.MODE_EXPLICIT);
        param.setCompressionQuality(compressionRatio);
        writer.write(null, new IIOImage(image, null, null), param);

        os.close();
        ios.close();
        writer.dispose();

        return new ByteArrayInputStream(os.toByteArray());
    }

    private InputStream checkFileSize(ImageDto imageDto, String minCompressingFileSize) throws IOException {
        Long minCompressingBytes = DataSize.parse(minCompressingFileSize).toBytes();
        Long resourceBytesLength = imageDto.getContentLength();
        if (resourceBytesLength <= minCompressingBytes) {
            return imageDto.getFileResource().getInputStream();
        }
        return null;
    }
}
