package edu.uoc.petfinder.spatial.impl;

import edu.uoc.petfinder.dto.ActionDto;
import edu.uoc.petfinder.dto.LocationDto;
import lombok.AllArgsConstructor;
import org.locationtech.spatial4j.context.SpatialContext;
import org.locationtech.spatial4j.distance.DistanceUtils;
import org.locationtech.spatial4j.shape.Point;
import org.locationtech.spatial4j.shape.ShapeFactory;
import org.springframework.data.geo.Distance;
import org.springframework.data.geo.Metrics;
import org.springframework.stereotype.Component;

import javax.validation.constraints.NotNull;

@Component
@AllArgsConstructor
public class SpatialOperations {

    private SpatialContext spatialContext;

    public Distance calculateDistanceBetweenTwoLocations(@NotNull LocationDto fromLoc, @NotNull LocationDto toLoc ) {
        ShapeFactory factory = this.spatialContext.getShapeFactory();
        Point fromPoint = factory.pointXY(fromLoc.getLongitude(), fromLoc.getLatitude());
        Point toPoint = factory.pointXY(toLoc.getLongitude(), toLoc.getLatitude());
        Double distValue = DistanceUtils.degrees2Dist(this.spatialContext.calcDistance(fromPoint, toPoint), DistanceUtils.EARTH_MEAN_RADIUS_KM);
        return new Distance(distValue, Metrics.KILOMETERS);
    }
}
