package edu.uoc.petfinder.spatial.impl;

import edu.uoc.petfinder.constant.ActionType;
import edu.uoc.petfinder.dto.LocationDto;
import edu.uoc.petfinder.model.Action;
import edu.uoc.petfinder.model.Pet;
import org.bson.BsonRegularExpression;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Example;
import org.springframework.data.domain.ExampleMatcher;
import org.springframework.data.domain.Pageable;
import org.springframework.data.geo.*;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.CriteriaDefinition;
import org.springframework.data.mongodb.core.query.NearQuery;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.data.mongodb.core.schema.MongoJsonSchema;
import org.springframework.stereotype.Repository;


@Repository
public class SpatialRepository {

    @Autowired
    private MongoTemplate mongoTemplate;

    public GeoPage<Pet> searchUnfoundPetsByLocation(LocationDto location, Pageable pageable) {
        Query baseQuery = new Query();
        baseQuery.addCriteria(Criteria.where("lastState.type").ne(ActionType.FOUND.name()));
        return this.buildMongoGeoQuery(location, pageable, baseQuery);
    }

    public GeoPage<Pet> searchFoundPetsByLocation(LocationDto location, Pageable pageable) {
        Query baseQuery = new Query();
        baseQuery.addCriteria(Criteria.where("lastState.type").is(ActionType.FOUND.name()));
        return this.buildMongoGeoQuery(location, pageable, baseQuery);
    }

    private GeoPage<Pet> buildMongoGeoQuery(LocationDto location, Pageable pageable, Query baseQuery) {
        Long total = mongoTemplate.count(baseQuery, Pet.class);
        NearQuery nearQuery = NearQuery.near(new Point(location.getLongitude(), location.getLatitude()), Metrics.KILOMETERS);
        nearQuery.spherical(true);
        nearQuery.query(baseQuery);
        nearQuery.with(pageable);
        final GeoResults<Pet> geoResults = mongoTemplate.geoNear(nearQuery, Pet.class);
        return new GeoPage<Pet>(geoResults, pageable, total);
    }
}
