package edu.uoc.petfinder.spatial.impl;

import edu.uoc.petfinder.dto.LocationDto;
import edu.uoc.petfinder.dto.SimplePetDto;
import edu.uoc.petfinder.mapper.ApiMapper;
import edu.uoc.petfinder.model.Pet;
import edu.uoc.petfinder.spatial.SpatialService;
import lombok.AllArgsConstructor;
import org.springframework.data.domain.Pageable;
import org.springframework.data.geo.GeoPage;
import org.springframework.data.geo.GeoResult;
import org.springframework.data.geo.GeoResults;
import org.springframework.stereotype.Service;

import java.util.stream.Collectors;

@Service
@AllArgsConstructor
public class SpatialServiceImpl implements SpatialService {

    private SpatialRepository spatialRepository;

    private ApiMapper mapper;

    @Override
    public GeoPage<SimplePetDto> searchSimplePetByLocation(LocationDto location, Pageable pageable) {
        final GeoPage<Pet> petGeoPage = this.spatialRepository.searchUnfoundPetsByLocation(location, pageable);
        GeoResults<SimplePetDto> geoResults = this.buildSimplePetGeoResults(petGeoPage);
        GeoPage<SimplePetDto> simplePetDtoGeoPage = new GeoPage<>(geoResults, petGeoPage.getPageable(), petGeoPage.getTotalElements());
        return simplePetDtoGeoPage;
    }

    @Override
    public GeoPage<SimplePetDto> searchFoundSimplePetByLocation(LocationDto location, Pageable pageable) {
        final GeoPage<Pet> petGeoPage = this.spatialRepository.searchFoundPetsByLocation(location, pageable);
        GeoResults<SimplePetDto> geoResults = this.buildSimplePetGeoResults(petGeoPage);
        GeoPage<SimplePetDto> simplePetDtoGeoPage = new GeoPage<>(geoResults, petGeoPage.getPageable(), petGeoPage.getTotalElements());
        return simplePetDtoGeoPage;
    }

    private GeoResults<SimplePetDto> buildSimplePetGeoResults(GeoPage<Pet> petGeoPage) {
        return new GeoResults<SimplePetDto>(petGeoPage
                .stream()
                .map(p -> (new GeoResult<SimplePetDto>(this.mapper.petToSimplePetDto(p.getContent()), p.getDistance()))
                ).collect(Collectors.toList()), petGeoPage.getAverageDistance());
    }
}
