package edu.uoc.petfinder.controller;

import edu.uoc.petfinder.constant.PetType;
import edu.uoc.petfinder.dto.SimplePetDto;
import edu.uoc.petfinder.dto.UserDto;
import edu.uoc.petfinder.service.PetService;
import edu.uoc.petfinder.service.UserService;
import org.junit.jupiter.api.Test;
import org.mockito.Mock;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.http.MediaType;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders;

import java.util.Arrays;

import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.BDDMockito.given;
import static org.hamcrest.Matchers.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.jsonPath;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;


class AdminControllerTest extends BaseControllerTest {

    @MockBean
    private final UserService userService = null;

    @MockBean
    private final PetService petService = null;


    @Test
    @WithMockUser(username = "email@email.es", roles = {"ADMIN"})
    void fetchUsers() throws Exception {
        //G
        String emailAsset = "email@email.com";
        super.mockAuthenticatedUserInjection(this.userService, emailAsset);

        UserDto user = UserDto.builder().email("abc@abc.es").password("psswrd").build();

        Page<UserDto> userDtos = new PageImpl<UserDto>(Arrays.asList(user, user));
        //W

        given(this.userService.searchUsers(anyString(), any())).willReturn(userDtos);

        //T
        super.mockMvc.perform(MockMvcRequestBuilders.get("/admin/users")
                .queryParam("search", "some thing")
                .contentType(MediaType.APPLICATION_JSON)
                .accept(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.content").exists())
                .andExpect(jsonPath("$.content", hasSize(2)));

        verify(this.userService, times(1)).searchUsers(anyString(), any());
    }

    @Test
    @WithMockUser(username = "email@email.es", roles = {"ADMIN"})
    void fetchPets() throws Exception {
        //G
        String emailAsset = "email@email.com";
        super.mockAuthenticatedUserInjection(this.userService, emailAsset);

        SimplePetDto petDto = new SimplePetDto();
        petDto.setName("Macho");
        petDto.setType(PetType.DOG);

        Page<SimplePetDto> petDtos = new PageImpl<>(Arrays.asList(petDto, petDto));

        //W
        given(this.petService.searchPets(anyString(), any())).willReturn(petDtos);

        //T
        super.mockMvc.perform(MockMvcRequestBuilders.get("/admin/pets")
                .queryParam("search", "some thing")
                .contentType(MediaType.APPLICATION_JSON)
                .accept(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.content").exists())
                .andExpect(jsonPath("$.content", hasSize(2)));

        verify(this.petService, times(1)).searchPets(anyString(), any());
    }

    @Test
    @WithMockUser(username = "email@email.es", roles = {"ADMIN"})
    void deletePet() throws Exception {
        //G
        String emailAsset = "email@email.com";
        super.mockAuthenticatedUserInjection(this.userService, emailAsset);

        //T
        super.mockMvc.perform(MockMvcRequestBuilders.delete("/admin/pet/{petId}", "1234")
                .contentType(MediaType.APPLICATION_JSON)
                .accept(MediaType.APPLICATION_JSON))
                .andExpect(status().isNoContent());

        verify(this.petService, times(1)).deletePet(anyString(), any(), any());
    }

    @Test
    @WithMockUser(username = "email@email.es", roles = {"ADMIN"})
    void deleteUser() throws Exception {
        //G
        String emailAsset = "email@email.com";
        super.mockAuthenticatedUserInjection(this.userService, emailAsset);

        //T
        super.mockMvc.perform(MockMvcRequestBuilders.delete("/admin/user/{userId}", "1234")
                .contentType(MediaType.APPLICATION_JSON)
                .accept(MediaType.APPLICATION_JSON))
                .andExpect(status().isNoContent());

        verify(this.userService, times(1)).deleteUser(anyString());
    }
}