package edu.uoc.petfinder.controller;

import edu.uoc.petfinder.dto.LoginResponseDto;
import edu.uoc.petfinder.dto.SignInUserDto;
import edu.uoc.petfinder.dto.TokenDto;
import edu.uoc.petfinder.dto.UserDto;
import edu.uoc.petfinder.security.AuthenticationService;
import org.junit.jupiter.api.Test;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders;

import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.BDDMockito.given;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.jsonPath;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;
import static org.hamcrest.Matchers.*;

class AuthControllerTest extends BaseControllerTest {

    @MockBean
    private AuthenticationService authenticationService;

    @Test
    void userSignIn() throws Exception {
        //G
        SignInUserDto dto = new SignInUserDto();
        dto.setEmail("abc@abc.es");
        dto.setPassword("psswrd");

        LoginResponseDto responseDto = LoginResponseDto.builder().email("abc@abc.es").token("TOKEN").build();
        //W
        given(this.authenticationService.signIn(any())).willReturn(responseDto);

        //T
        super.mockMvc.perform(MockMvcRequestBuilders.post("/auth/signin")
                .content(super.objectMapper.writeValueAsString(dto))
                .contentType(MediaType.APPLICATION_JSON)
                .accept(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.email", is("abc@abc.es")))
                .andExpect(jsonPath("$.token", is("TOKEN")));
    }

    @Test
    void userSignUp()throws Exception {
        //G
        UserDto dto = new UserDto();
        dto.setEmail("abc@abc.es");
        dto.setPassword("psswrd");

        SignInUserDto signInUserDto = new SignInUserDto();
        signInUserDto.setEmail("abc@abc.es");
        signInUserDto.setPassword("psswrd");

        LoginResponseDto responseDto = LoginResponseDto.builder().email("abc@abc.es").token("TOKEN").build();
        //W
        given(this.authenticationService.signUp(any())).willReturn(responseDto);

        //T
        super.mockMvc.perform(MockMvcRequestBuilders.post("/auth/signup")
                .content(super.objectMapper.writeValueAsString(signInUserDto))
                .contentType(MediaType.APPLICATION_JSON)
                .accept(MediaType.APPLICATION_JSON))
                .andExpect(status().isCreated())
                .andExpect(jsonPath("$.email", is("abc@abc.es")))
                .andExpect(jsonPath("$.token", is("TOKEN")));
    }

    @Test
    void facebookSignIn() throws Exception {
        //G
        TokenDto tokenDto = new TokenDto();
        tokenDto.setToken("TOKEN");

        LoginResponseDto responseDto = LoginResponseDto.builder().email("abc@abc.es").token("TOKEN").build();
        //W
        given(this.authenticationService.connectWithFacebook(any())).willReturn(responseDto);

        //T
        super.mockMvc.perform(MockMvcRequestBuilders.post("/auth/facebook")
                .content(super.objectMapper.writeValueAsString(tokenDto))
                .contentType(MediaType.APPLICATION_JSON)
                .accept(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.email", is("abc@abc.es")))
                .andExpect(jsonPath("$.token", is("TOKEN")));
    }

    @Test
    void googleSignIn() throws Exception {
        //G
        TokenDto tokenDto = new TokenDto();
        tokenDto.setToken("TOKEN");

        LoginResponseDto responseDto = LoginResponseDto.builder().email("abc@abc.es").token("TOKEN").build();
        //W
        given(this.authenticationService.connectWithGoogle(any())).willReturn(responseDto);

        //T
        super.mockMvc.perform(MockMvcRequestBuilders.post("/auth/google")
                .content(super.objectMapper.writeValueAsString(tokenDto))
                .contentType(MediaType.APPLICATION_JSON)
                .accept(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.email", is("abc@abc.es")))
                .andExpect(jsonPath("$.token", is("TOKEN")));
    }
}