package edu.uoc.petfinder.controller;

import edu.uoc.petfinder.dto.ImageDto;
import edu.uoc.petfinder.service.ImageService;
import org.junit.jupiter.api.Test;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.core.io.InputStreamResource;
import org.springframework.core.io.Resource;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders;

import java.io.ByteArrayInputStream;
import java.io.InputStream;

import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.ArgumentMatchers.anyString;
import static org.mockito.BDDMockito.given;
import static org.hamcrest.Matchers.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

class ImageControllerTest extends BaseControllerTest {

    @MockBean
    private ImageService imageService;

    @Test
    void retrieveImageResponseEntity() throws Exception {
        //G
        InputStream inputStream;
        Resource resource = new InputStreamResource(new ByteArrayInputStream(new String("original").getBytes()));
        ImageDto imageDto = ImageDto.builder().id("abc123").contentType("image/png").fileResource(resource).contentLength(2343243L).build();

        //W
        given(this.imageService.fetchImage(anyString(), any())).willReturn(imageDto);

        //T
        super.mockMvc.perform(MockMvcRequestBuilders.get("/image/{testName}", "text.png")
                .contentType(MediaType.APPLICATION_JSON)
                .accept(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(header().string("Content-Type", "image/png"));
    }
}