import { Component, OnInit } from '@angular/core';
import { SocialAuthService } from "angularx-social-login";
import { FacebookLoginProvider, GoogleLoginProvider } from "angularx-social-login";
import { BsModalRef } from 'ngx-bootstrap/modal';
import { switchMap, takeUntil } from 'rxjs/operators';
import { AuthService } from 'src/app/services/auth.service';
import User from 'src/app/models/User';
import { BaseComponent } from 'src/app/shared/base.component';
import { from } from 'rxjs';

enum AccessModalContent {
  LOGIN = 'LOGIN',
  REGISTER = 'REGISTER'
}

@Component({
  selector: 'app-access-modal',
  templateUrl: './access-modal.component.html',
  styleUrls: ['./access-modal.component.scss']
})
export class AccessModalComponent extends BaseComponent implements OnInit {

  AccessModalContent = AccessModalContent;

  contentType = AccessModalContent.LOGIN

  public spinnerVisible = false;

  public connectionError = false;

  constructor(public accessModalRef: BsModalRef, private authService: AuthService, private socialLoginService: SocialAuthService) {
    super();
  }

  ngOnInit(): void {
    this.accessModalRef.onHide.subscribe(() => this.ngOnDestroy())
  }

  signup(user: User) {
    this.spinnerVisible = true;
    this.authService.signup(user).pipe(
      takeUntil(this.destroy$)
    ).subscribe(() => this.completeSuccessLogin(), () => this.handleError())
  }

  login(user: User) {
    this.spinnerVisible = true;
    this.authService.login(user).pipe(
      takeUntil(this.destroy$)
    ).subscribe(() => this.completeSuccessLogin(), () => this.handleError())
  }

  private completeSuccessLogin() {
    this.spinnerVisible = false;
    this.accessModalRef.hide();
  }

  async loginWithFacebook() {
    this.spinnerVisible = true;
    from(this.socialLoginService.signIn(FacebookLoginProvider.PROVIDER_ID)).pipe(
      switchMap((user) => this.authService.facebookLogin(user.authToken).pipe(
        takeUntil(this.destroy$)
      ))
    ).subscribe(() => this.completeSuccessLogin(), () => this.handleError())
  }

  loginWithGoogle() {
    this.spinnerVisible = true;
    from(this.socialLoginService.signIn(GoogleLoginProvider.PROVIDER_ID)).pipe(
      switchMap((user) => this.authService.googleLogin(user.idToken).pipe(
        takeUntil(this.destroy$)
      ))
    ).subscribe(() => this.completeSuccessLogin(), () => this.handleError())
  }

  private handleError() {
    this.spinnerVisible = false;
    this.connectionError = true;
  }

}
