import { ChangeDetectionStrategy, Component, EventEmitter, OnDestroy, OnInit, Output } from '@angular/core';
import { FormBuilder, FormControl, FormGroup, Validators } from '@angular/forms';
import User from 'src/app/models/User';

@Component({
  selector: 'app-login-form',
  templateUrl: './login-form.component.html',
  styleUrls: ['./login-form.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class LoginFormComponent implements OnInit, OnDestroy {

  public loginForm: FormGroup

  @Output()
  public login = new EventEmitter();

  constructor(private formBuilder: FormBuilder) { }

  ngOnInit(): void {
    this.loginForm = this.formBuilder.group({
      email: this.formBuilder.control('', [Validators.required, Validators.email]),
      password: this.formBuilder.control('', [Validators.required])
    })
  }

  submitLoginForm() {
    const loginUser: User = this.loginForm.value;
    this.login.emit(loginUser);
  }

  ngOnDestroy() {
    this.loginForm.reset();
  }

  get emailInput(): FormControl {
    return this.loginForm.get('email') as FormControl;
  }

  get passwordInput(): FormControl {
    return this.loginForm.get('password') as FormControl;
  }

}
