import { AfterViewInit, Component, OnInit, ViewChild } from '@angular/core';
import { LeafletDirective } from '@asymmetrik/ngx-leaflet';
import { LatLng, latLng, MapOptions, tileLayer } from 'leaflet';
import { BsModalRef } from 'ngx-bootstrap/modal';
import { takeUntil } from 'rxjs/operators';
import { Action } from 'src/app/models/Action';
import { BaseComponent } from 'src/app/shared/base.component';
import LeafletFactory from 'src/app/shared/LeafletFactory';
import { environment } from 'src/environments/environment';
import { ActionsService } from '../actions.service';

@Component({
  selector: 'app-action-detail-modal',
  templateUrl: './action-detail-modal.component.html',
  styleUrls: ['./action-detail-modal.component.scss'],
})
export class ActionDetailModalComponent
  extends BaseComponent
  implements OnInit, AfterViewInit {
  action: Action;

  options: MapOptions;

  @ViewChild(LeafletDirective)
  map: LeafletDirective;

  constructor(private actionsService: ActionsService, public bsModalRef: BsModalRef) {
    super();
  }

  ngOnInit(): void {
    this.actionsService.actionDetail$
      .pipe(takeUntil(this.destroy$))
      .subscribe((a) => (this.action = a));

    this.options = {
      layers: [tileLayer(environment.maps_provider, { maxZoom: 18 })],
      zoom: 13,
      center: latLng(this.action.location.lat, this.action.location.long),
    };
  }

  ngAfterViewInit() {
    setTimeout(() => {
      this.map.getMap().invalidateSize();
      this.map.getMap().addLayer(LeafletFactory.marker(new LatLng(this.action.location.lat, this.action.location.long)))
    }, 100);
  }
}
