import { Injectable } from '@angular/core';
import { ReplaySubject } from 'rxjs';
import { tap } from 'rxjs/operators';
import { Action } from 'src/app/models/Action';
import { ActionType } from 'src/app/models/ActionType';
import Pet from 'src/app/models/Pet';
import { AnalyticsService } from 'src/app/services/analytics.service';
import { RestService } from 'src/app/services/rest.service';
import { environment } from 'src/environments/environment';
import { Location } from '../../models/Location';

@Injectable({
  providedIn: 'root',
})
export class ActionsService {
  private petSbjct = new ReplaySubject<{
    pet: Pet;
    action: ActionType;
    location: Location;
  }>(1);

  private actionDetailSbjct = new ReplaySubject<Action>(1);

  constructor(private rest: RestService, private analytics: AnalyticsService) {}

  setPetAndAction(pet: Pet, action: ActionType, location: Location) {
    this.petSbjct.next({ pet, action, location });
  }

  setActionToDetail(action:Action) {
    this.actionDetailSbjct.next(action);
  }

  get actionDetail$() {
    return this.actionDetailSbjct.asObservable();
  }

  get pet$() {
    return this.petSbjct.asObservable();
  }

  createActionInPet(petId: string, actionType: ActionType, action: Action) {
    switch (actionType) {
      case ActionType.SIGHTING:
        return this.rest
          .post(`/pet/${petId}/sighting`, action)
          .pipe(tap(() => this.analytics.petAction(action)));

      case ActionType.FOUND:
        return this.rest
          .post(`/pet/${petId}/found`, action)
          .pipe(tap(() => this.analytics.petAction(action)));
      case ActionType.LOST:
        return this.rest
          .post(`/pet/${petId}/lost`, action)
          .pipe(tap(() => this.analytics.petAction(action)));

      default:
        throw new Error('Error: unhandled actiontype.');
    }
  }
}
