import { Component, OnInit } from '@angular/core';
import { TranslateService } from '@ngx-translate/core';
import { take, map } from 'rxjs/operators';
import Page from 'src/app/models/Page';
import { PageParams } from 'src/app/models/PageParams';
import Pet from 'src/app/models/Pet';
import User from 'src/app/models/User';
import { NotificationService } from 'src/app/services/notification.service';
import { UserService } from 'src/app/services/user.service';
import { BaseComponent } from 'src/app/shared/base.component';
import { PetService } from '../../pet/pet.service';

@Component({
  selector: 'app-admin-pets',
  templateUrl: './admin-pets.component.html',
  styleUrls: ['./admin-pets.component.scss']
})
export class AdminPetsComponent extends BaseComponent implements OnInit {

  searchTerm: string;

  pets: Pet[];

  page: Page<Pet>;

  constructor(
    private petService: PetService,
    private notificationService: NotificationService,
    private translate: TranslateService
  ) {
    super();
  }

  ngOnInit(): void {
    this.fetchPets();
  }

  fetchPets(page = 0) {
    this.petService
      .searchPets(this.searchTerm, { page })
      .pipe(take(1))
      .subscribe(
        (petsPage: Page<Pet>) => {
          this.page = petsPage;
          this.pets = petsPage.content as Pet[];
        },
        () =>
          this.notificationService.error(
            this.translate.instant('response.ko.loadingPets')
          )
      );
  }

  delete(pet: Pet) {
    this.petService
      .deletePet(pet)
      .pipe(take(1))
      .subscribe(
        () => {
          this.fetchPets();
          this.notificationService.success(
            this.translate.instant('response.ok.deletePet', {
              name: pet.name,
            })
          );
        },
        () =>
          this.notificationService.error(
            this.translate.instant('response.ko.deletePet')
          )
      );
  }

  paginate(page: PageParams) {
    this.fetchPets(page.page);
  }

  onSearch() {
    this.fetchPets();
  }

}
