import { Component, OnInit } from '@angular/core';
import { TranslateService } from '@ngx-translate/core';
import { map, take, tap } from 'rxjs/operators';
import Page from 'src/app/models/Page';
import { PageParams } from 'src/app/models/PageParams';
import User from 'src/app/models/User';
import { NotificationService } from 'src/app/services/notification.service';
import { UserService } from 'src/app/services/user.service';
import { BaseComponent } from 'src/app/shared/base.component';

@Component({
  selector: 'app-admin-users',
  templateUrl: './admin-users.component.html',
  styleUrls: ['./admin-users.component.scss'],
})
export class AdminUsersComponent extends BaseComponent implements OnInit {
  searchTerm: string;

  users: User[];

  page: Page<User>;

  constructor(
    private userService: UserService,
    private notificationService: NotificationService,
    private translate: TranslateService
  ) {
    super();
  }

  ngOnInit(): void {
    this.fetchUsers();
  }

  fetchUsers(page = 0) {
    this.userService
      .searchUsers(this.searchTerm, { page })
      .pipe(take(1))
      .subscribe(
        (userPage: Page<User>) => {
          this.page = userPage;
          this.users = userPage.content as User[];
        },
        () =>
          this.notificationService.error(
            this.translate.instant('response.ko.loadingUsers')
          )
      );
  }

  delete(user: User) {
    this.userService
      .deleteUser(user)
      .pipe(take(1))
      .subscribe(
        () => {
          this.fetchUsers();
          this.notificationService.success(
            this.translate.instant('response.ok.deleteUser', {
              email: user.email,
            })
          );
        },
        () =>
          this.notificationService.error(
            this.translate.instant('response.ko.deleteUser')
          )
      );
  }

  paginate(page: PageParams) {
    this.fetchUsers(page.page);
  }

  canBeDeleted(inputUser: User) {
    return this.userService.user.pipe(
      map((user: User) => {
        if (
          user?.email === inputUser.email ||
          inputUser.roles?.some((r) => r.name === 'ROLE_ADMIN')
        ) {
          return false;
        }
        return true;
      })
    );
  }

  onSearch() {
    this.fetchUsers();
  }
}
