import { NgModule } from '@angular/core';
import { Routes, RouterModule } from '@angular/router';
import { IsAdminGuard } from '../guards/is-admin.guard';
import { IsAuthenticatedGuard } from '../guards/is-authenticated.guard';
import { PetResolver } from '../resolvers/pet.resolver';
import { ContentComponent } from './content.component';
import { HomeComponent } from './home/home.component';
import { PetDetailComponent } from './pet/pet-detail/pet-detail.component';


const routes: Routes = [
  { path: '', component: ContentComponent, children: [
    { path: 'home', component: HomeComponent },
    { path: 'pet/:id', component: PetDetailComponent, resolve: {pet: PetResolver} },
    { path: 'user', canLoad: [IsAuthenticatedGuard], loadChildren: () => import('./user/user.module').then(m => m.UserModule) },
    { path: 'found', loadChildren: () => import('./found/found.module').then(m => m.FoundModule) },
    { path: 'legal-note', loadChildren: () => import('./legal-note/legal-note.module').then(m => m.LegalNoteModule) },
    { path: 'about-me', loadChildren: () => import('./about-me/about-me.module').then(m => m.AboutMeModule) },
    { path: 'manage', canLoad: [IsAuthenticatedGuard, IsAdminGuard], loadChildren: () => import('./admin/admin.module').then(m => m.AdminModule) },
    { path: '', pathMatch: 'full', component: HomeComponent }
  ]},
];

@NgModule({
  imports: [RouterModule.forChild(routes)],
  exports: [RouterModule]
})
export class ContentRoutingModule { }
