import { Injectable } from '@angular/core';
import { Title } from '@angular/platform-browser';
import { NavigationEnd, NavigationStart, Router } from '@angular/router';
import { BehaviorSubject } from 'rxjs';

@Injectable({
  providedIn: 'root'
})
export class LayoutService {

  private extendedHeaderSbjct = new BehaviorSubject(true);

  private openedSidebarSbjct = new BehaviorSubject(false);

  constructor(private router: Router, private title: Title) {
    this.manageNavigationEvents();
  }

  private manageNavigationEvents() {
    this.router.events.subscribe(e => {
      if (e instanceof NavigationStart) {
        this.toggleSidebar(false)
      }
      if (e instanceof NavigationEnd) {
        window.scrollTo(0,0);
        this.extendedHeaderSbjct.next(true);
        this.title.setTitle('Petfinder');
      }
    });
  }

  get openedSidebar$() {
    return this.openedSidebarSbjct.asObservable();
  }

  get extendedHeader$() {
    return this.extendedHeaderSbjct.asObservable();
  }

  setExtendedHeader(isExtended: boolean) {
    this.extendedHeaderSbjct.next(isExtended);
  }

  toggleSidebar(isOpened?: boolean) {
    if (isOpened !== null && isOpened !== undefined) {
      this.openedSidebarSbjct.next(isOpened);
    } else {
      this.openedSidebarSbjct.next(!this.openedSidebarSbjct.getValue());
    }
  }

  setPageTitle(title: string) {
    if (title) {
      this.title.setTitle(`PetFinder - ${title}`);
    } else {
      this.title.setTitle(`PetFinder`);
    }

  }
}
