import { Component, Input, OnInit } from '@angular/core';
import { stringify } from 'querystring';
import GeoContent from 'src/app/models/GeoContent';
import { Image } from 'src/app/models/Image';
import Pet from 'src/app/models/Pet';
import { PetType } from 'src/app/models/PetType';
import { environment } from 'src/environments/environment';
import { PetContentType } from '../../home/home.component';

@Component({
  selector: 'app-pet-card',
  templateUrl: './pet-card.component.html',
  styleUrls: ['./pet-card.component.scss']
})
export class PetCardComponent implements OnInit {

  @Input()
  data;

  @Input()
  contentType: PetContentType

  @Input()
  isFound = false;

  @Input()
  isEditable = false;


  fetchGeoPet: GeoContent<Pet>;

  fetchedPet: Pet;

  PetContentType = PetContentType;

  PetType = PetType;

  constructor() { }

  ngOnInit(): void {
    this.contentType === PetContentType.GEOCONTENT ? this.fetchGeoPet = this.data : this.fetchedPet = this.data;
  }

  getPetImage(): string {
    let image: Image ;
    if (this.fetchedPet) {
      image = this.fetchedPet.images ? this.fetchedPet.images[0] : null;
    } else {
      image =  this.fetchGeoPet.content.images ? this.fetchGeoPet.content.images[0] : null;
    }
    return !!image ? `${environment.api_endpoint}/image/${image.systemFileName}` : 'assets/images/no-image.svg';

  }

  get petType(): PetType {
    return this.fetchedPet?.type || this.fetchGeoPet?.content.type;
  }

  get pet() {
    return this.fetchedPet || this.fetchGeoPet?.content;
  }

}
