import {
  ChangeDetectionStrategy,
  Component,
  Input,
  OnInit,
} from '@angular/core';
import { Router } from '@angular/router';
import { combineLatest, Observable } from 'rxjs';
import { map, take } from 'rxjs/operators';
import { AccessModalService } from 'src/app/content/access/access-modal.service';
import { ActionsModalService } from 'src/app/content/actions/actions-modal.service';
import { ActionsService } from 'src/app/content/actions/actions.service';
import { ActionType } from 'src/app/models/ActionType';
import Pet from 'src/app/models/Pet';
import { GeolocationService } from 'src/app/services/geolocation.service';
import { UserService } from 'src/app/services/user.service';
import { BaseComponent } from 'src/app/shared/base.component';
import { Location } from 'src/app/models/Location';

@Component({
  selector: 'app-pet-actions',
  templateUrl: './pet-actions.component.html',
  styleUrls: ['./pet-actions.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class PetActionsComponent extends BaseComponent implements OnInit {
  @Input()
  pet: Pet;

  ActionType = ActionType;

  constructor(
    private accessModalService: AccessModalService,
    private userService: UserService,
    private geolocationService: GeolocationService,
    private actionsModalService: ActionsModalService
  ) {
    super();
  }

  ngOnInit(): void {}

  canLose(): Observable<boolean> {
    return this.userService.user.pipe(
      map(user => user.email === this.pet.owner.email ? true : false)
    );
  }

  launchModal(evt: Event, actionType: ActionType) {
    evt.stopPropagation();
    const canLaunch = combineLatest([
      this.geolocationService.location as Observable<Location>,
      this.userService.isAuthenticated,
    ]);
    canLaunch
      .pipe(take(1))
      .subscribe(([location, isAuth]) =>
        isAuth
          ? this.actionsModalService.openActionsModal(this.pet, actionType, location)
          : this.accessModalService.openAccessModal()
      );
  }
}
