import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { ActivatedRoute, Router } from '@angular/router';
import { TranslateService } from '@ngx-translate/core';
import { take } from 'rxjs/operators';
import User from 'src/app/models/User';
import { NotificationService } from 'src/app/services/notification.service';
import { UserService } from 'src/app/services/user.service';
import { BaseComponent } from 'src/app/shared/base.component';
import { LayoutService } from '../../layout/layout.service';

@Component({
  selector: 'app-user-detail',
  templateUrl: './user-detail.component.html',
  styleUrls: ['./user-detail.component.scss'],
})
export class UserDetailComponent extends BaseComponent implements OnInit {
  userForm: FormGroup;

  user: User;

  isEditable = false;

  constructor(
    private layoutService: LayoutService,
    private route: ActivatedRoute,
    private fb: FormBuilder,
    private userService: UserService,
    private notificationService: NotificationService,
    private router: Router,
    private translate: TranslateService
  ) {
    super();
  }

  ngOnInit(): void {
    this.layoutService.setExtendedHeader(false);
    this.user = this.route.snapshot.data.user;
    this.layoutService.setPageTitle(this.translate.instant('pageTitle.userDetail'));
    this.userForm = this.fb.group({
      firstName: this.fb.control(
        { value: this.user.firstName, disabled: !this.isEditable },
        [Validators.required]
      ),
      lastName: this.fb.control(
        { value: this.user.lastName, disabled: !this.isEditable },
        [Validators.required]
      ),
      email: this.fb.control({ value: this.user.email, disabled: true }, [
        Validators.required,
        Validators.email,
      ]),
      birthDate: this.fb.control({
        value: this.user.birthDate,
        disabled: !this.isEditable,
      }),
    });
  }

  toggleEditable() {
    this.isEditable = !this.isEditable;
    const editableFields = [
      this.userForm.get('firstName'),
      this.userForm.get('lastName'),
      this.userForm.get('birthDate'),
    ];
    if (this.isEditable) {
      editableFields.forEach(control => control.enable())
    } else {
      this.userForm.reset(this.user);
      editableFields.forEach(control => control.disable())
    }
  }

  onSubmit() {
    this.userService.updateUser({...this.user, ...this.userForm.value}).pipe(take(1)).subscribe(() => {
      this.notificationService.success(this.translate.instant('response.ok.userUpdated'));
      this.router.navigate(['/'])
    }, () => this.notificationService.error(this.translate.instant('response.ko.userUpdated')));
  }
}
