import { Component, EventEmitter, Input, OnInit, Output } from '@angular/core';
import { FormBuilder, FormControl, FormGroup, Validators } from '@angular/forms';
import { Genre } from 'src/app/models/Genre';
import Pet from 'src/app/models/Pet';
import { PetType } from 'src/app/models/PetType';
import User from 'src/app/models/User';

@Component({
  selector: 'app-update-pet-form',
  templateUrl: './update-pet-form.component.html',
  styleUrls: ['./update-pet-form.component.scss']
})
export class UpdatePetFormComponent implements OnInit {

  PetType = PetType;
  Genre = Genre;

  @Input()
  loading = false;

  @Input()
  pet: Pet;

  @Output()
  onSave = new EventEmitter<User>();

  updatePetForm: FormGroup;

  constructor(private fb: FormBuilder) { }

  ngOnInit(): void {
    this.updatePetForm = this.fb.group({
      name: this.fb.control(this.pet.name, [Validators.required]),
      type: this.fb.control(this.pet.type, [Validators.required]),
      genre: this.fb.control(this.pet.genre, [Validators.required]),
      description: this.fb.control(this.pet.description, [Validators.maxLength(150)]),
      birthDate: this.fb.control(this.pet.birthDate),
    });
  }

  submitUpdatePetForm() {
    this.onSave.emit({...this.pet, ...this.updatePetForm.value});
    this.updatePetForm.reset({...this.pet, ...this.updatePetForm.value});
  }

  reset() {
    this.updatePetForm.reset(this.pet);
  }

  get name(): FormControl {
    return this.updatePetForm.get(['name']) as FormControl;
  }

  get type(): FormControl {
    return this.updatePetForm.get(['type']) as FormControl;
  }

  get genre(): FormControl {
    return this.updatePetForm.get(['genre']) as FormControl;
  }

  get description(): FormControl {
    return this.updatePetForm.get(['description']) as FormControl;
  }

}
