import { Component, EventEmitter, Input, OnInit, Output, ViewChild } from '@angular/core';
import { NgFileSelectDirective, UploadFile } from 'ngx-uploader';
import { Image } from 'src/app/models/Image';
import Pet from 'src/app/models/Pet';

@Component({
  selector: 'app-update-pet-images',
  templateUrl: './update-pet-images.component.html',
  styleUrls: ['./update-pet-images.component.scss']
})
export class UpdatePetImagesComponent implements OnInit {

  @Input()
  loading = false;

  @Input()
  pet: Pet;

  @Output()
  onSaveImage = new EventEmitter<UploadFile[]>();

  @Output()
  onDeleteImage = new EventEmitter<Image>();

  @ViewChild(NgFileSelectDirective)
  imageSelector: NgFileSelectDirective;

  files: UploadFile[] = [];

  selectedImage: Image;

  imageSelectorError = false;
  spinnerVisible = false;

  constructor() { }

  ngOnInit(): void {
    this.selectedImage = this.pet.images ? this.pet.images[0] : null ;
  }

  isSelected(image: Image) {
    return this.selectedImage?.id === image.id ? true : false;
  }

  onUploadImages(event) {
    if (event.type === 'rejected') {
      this.imageSelectorError = true;
    }
    if (event.type === 'addedToQueue') {
      this.files.push(event.file);
    }
  }

  delete(event: Event, image: Image) {
    event.stopPropagation();
    this.onDeleteImage.emit(image);
  }

  save() {
    this.onSaveImage.emit(this.files);
  }

  getUploaderOptions() {
    return {
      allowedContentTypes: ['image/jpeg', 'image/png'],
      concurrency: 5 - (this.pet.images?.length || 0),
      maxUploads: 5 - (this.pet.images?.length || 0),
      maxFileSize: 4000000 // 4MB
    }
  }

  resetImagesInput() {
    this.files = [];
    this.imageSelector.upload.queue = [];
    this.imageSelectorError = false;
  }
}
