import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormControl, FormGroup, Validators } from '@angular/forms';
import { ActivatedRoute } from '@angular/router';
import { Genre } from 'src/app/models/Genre';
import Pet from 'src/app/models/Pet';
import { Image } from 'src/app/models/Image';
import { PetType } from 'src/app/models/PetType';
import { NotificationService } from 'src/app/services/notification.service';
import { LayoutService } from '../../layout/layout.service';
import { PetService } from '../../pet/pet.service';
import { UploadFile } from 'ngx-uploader';
import { BaseComponent } from 'src/app/shared/base.component';
import { take } from 'rxjs/operators';
import { TranslateService } from '@ngx-translate/core';

@Component({
  selector: 'app-user-pet-edit',
  templateUrl: './user-pet-edit.component.html',
  styleUrls: ['./user-pet-edit.component.scss'],
})
export class UserPetEditComponent extends BaseComponent implements OnInit {

  pet: Pet;
  petFormLoading = false;
  petImagesLoading = false;

  constructor(
    private layoutService: LayoutService,
    private route: ActivatedRoute,
    private petService: PetService,
    private notificationService: NotificationService,
    private translate: TranslateService
  ) {
    super();
  }

  ngOnInit(): void {
    this.layoutService.setExtendedHeader(false);
    this.pet = this.route.snapshot.data.pet;
    this.layoutService.setPageTitle(this.translate.instant('pageTitle.petEdit', {name: this.pet.name}));
  }

  updatePetData(pet: Pet) {
    this.petFormLoading = true;
    this.petService.updatePet(pet).pipe(take(1)).subscribe(pet => {
      this.pet = pet;
      this.notificationService.success(this.translate.instant('response.ok.petUpdated'))
    }, () => this.notificationService.error(this.translate.instant('response.ko.petUpdated')), () => this.petFormLoading = false)
  }

  saveImage(files: UploadFile[]) {
    this.petImagesLoading = true;
    this.petService.addImagesToPet(this.pet, files).pipe(take(1)).subscribe(pet => {
      this.pet = pet;
      this.notificationService.success(this.translate.instant('response.ok.imageUploaded'))
    }, () => this.notificationService.error(this.translate.instant('response.ko.imageUploaded')), () => this.petImagesLoading = false)
  }

  deleteImage(image: Image) {
    this.petImagesLoading = true;
    this.petService.deletePetImage(this.pet, image).pipe(take(1)).subscribe(pet => {
      this.pet = pet;
      this.notificationService.success(this.translate.instant('response.ok.imageDeleted', {fileName: image.systemFileName}))
    }, () => this.notificationService.error(this.translate.instant('response.ko.imageDeleted')), () => this.petImagesLoading = false)
  }

}
