import { ChangeDetectionStrategy, Component, EventEmitter, Input, OnInit, Output } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { TranslateService } from '@ngx-translate/core';
import { map,  take,  tap } from 'rxjs/operators';
import { ActionType } from 'src/app/models/ActionType';
import { Location } from 'src/app/models/Location';
import Pet from 'src/app/models/Pet';
import { PetType } from 'src/app/models/PetType';
import { GeolocationService } from 'src/app/services/geolocation.service';
import { ActionsModalService } from '../../actions/actions-modal.service';

@Component({
  selector: 'app-user-pet-item',
  changeDetection: ChangeDetectionStrategy.OnPush,
  templateUrl: './user-pet-item.component.html',
  styleUrls: ['./user-pet-item.component.scss']
})
export class UserPetItemComponent implements OnInit {

  PetType = PetType;
  ActionType = ActionType;

  @Input()
  pet: Pet;

  @Output()
  delete = new EventEmitter();

  constructor(private router: Router, private route: ActivatedRoute, public translate: TranslateService, private actionsModalService: ActionsModalService, private location: GeolocationService) { }

  ngOnInit(): void {
  }

  goToEdit() {
    this.router.navigate(['..', 'pet', this.pet.id, 'edit'], {relativeTo: this.route });
  }

  launchLostModal() {
    this.location.location.pipe(take(1)).subscribe((location: Location) => {
      this.actionsModalService.openActionsModal(this.pet, ActionType.LOST, location);
    })
  }

  goToDetail() {
    this.router.navigate(['/', 'pet', this.pet.id]);
  }

  onDelete() {
    this.delete.emit(this.pet);
  }

}
