import { Component, OnInit } from '@angular/core';
import { TranslateService } from '@ngx-translate/core';
import { take, takeUntil } from 'rxjs/operators';
import Page from 'src/app/models/Page';
import Pet from 'src/app/models/Pet';
import { NotificationService } from 'src/app/services/notification.service';
import { BaseComponent } from 'src/app/shared/base.component';
import { LayoutService } from '../../layout/layout.service';
import { PetService } from '../../pet/pet.service';

@Component({
  selector: 'app-user-pets',
  templateUrl: './user-pets.component.html',
  styleUrls: ['./user-pets.component.scss'],
})
export class UserPetsComponent extends BaseComponent implements OnInit {
  isLoading = false;
  page = 0;
  petPage: Page<Pet>;

  pets: Pet[] = [];

  constructor(
    private layoutService: LayoutService,
    private petService: PetService,
    private notificationService: NotificationService,
    private translate: TranslateService
  ) {
    super();
  }

  ngOnInit(): void {
    this.layoutService.setExtendedHeader(false);
    this.layoutService.setPageTitle(this.translate.instant('pageTitle.userPets'));
    this.fetchOwnPets(false);
  }

  pageMoreElements() {
    if (this.petPage.last || this.petPage.empty) {
      return;
    }
    this.page++;
    this.fetchOwnPets(true);
  }

  private fetchOwnPets(isPaging = false) {
    this.isLoading = true;
    this.petService
      .fetchOwnPets({ page: this.page })
      .pipe(takeUntil(this.destroy$))
      .subscribe(
        (data) => {
          this.petPage = data;
          isPaging
            ? data.content.forEach((pet) => this.pets.push(pet))
            : (this.pets = data.content as Pet[]);
          this.isLoading = false;
        },
        () => {
          this.notificationService.error(
            this.translate.instant('response.ko.loadingPets')
          );
          this.isLoading = false;
        }
      );
  }

  deletePet(pet: Pet) {
    this.petService.deleteUserPet(pet).pipe(take(1)).subscribe(() => {
      this.fetchOwnPets(false);
      this.notificationService.success(this.translate.instant('response.ok.deletePet', {name: pet.name}))
    }, () => this.notificationService.error(this.translate.instant('response.ko.deletePet', {name: pet.name})));
  }

  get gridTitle() {
    return this.translate.instant('userPets.title');
  }
}
