import { Injectable } from '@angular/core';
import { CanActivate, ActivatedRouteSnapshot, RouterStateSnapshot, UrlTree, Router } from '@angular/router';
import { forkJoin, Observable, throwError } from 'rxjs';
import { catchError, combineAll, map, tap, withLatestFrom, zip } from 'rxjs/operators';
import { PetService } from '../content/pet/pet.service';
import { UserService } from '../services/user.service';

@Injectable({
  providedIn: 'root'
})
export class IsOwnPetGuard implements CanActivate {

  constructor(private petService: PetService, private userService: UserService, private router: Router) {}

  canActivate(
    route: ActivatedRouteSnapshot,
    state: RouterStateSnapshot): Observable<boolean | UrlTree>  {
    const id = route.params['id'];
    return this.petService.getPetDetail(id).pipe(
      withLatestFrom(this.userService.user),
      map(([pet, user]) => pet.owner.email !== user.email ? this.router.createUrlTree(['/']) : true),
      catchError(err => {
        this.router.navigate(['/']);
        return throwError(err);
      })
    )
  }

}
