import { Injectable } from '@angular/core';
import { Resolve, ActivatedRouteSnapshot, Router } from '@angular/router';
import { Observable, throwError } from 'rxjs';
import { catchError, map, switchMap, take, tap } from 'rxjs/operators';
import { PetService } from '../content/pet/pet.service';
import Pet from '../models/Pet';
import User from '../models/User';
import { UserService } from '../services/user.service';

@Injectable({ providedIn: 'root' })
export class UserResolver implements Resolve<User> {

  constructor(private userService: UserService, private router: Router) {}

  resolve(): Observable<User> {
    return this.userService.user.pipe(
      map(user => user.email),
      take(1),
      switchMap((email: string) => this.userService.fetchUser(email).pipe(
        tap(user => this.userService.setUser(user))
      ))
    )
  }
}
