import { Injectable } from '@angular/core';
import { Title } from '@angular/platform-browser';
import { NavigationEnd, Router } from '@angular/router';
import { Action } from '../models/Action';
import Pet from '../models/Pet';

declare let gtag: Function;

export enum LoginType {
  LOCAL,
  FACEBOOK,
  GOOGLE,
}

@Injectable({
  providedIn: 'root',
})
export class AnalyticsService {
  constructor(public router: Router, private title: Title) {
    this.sendPageView();
  }

  private sendPageView() {
    this.router.events.subscribe((event) => {
      if (event instanceof NavigationEnd) {
        setTimeout(() => {
          gtag('config', 'G-1SSCGX8MZV', {
            page_path: event.urlAfterRedirects,
            page_title: this.title.getTitle(),
          });
        }, 200);
      }
    });
  }

  login(loginMethod: LoginType) {
    this.eventEmitter('login', 'engagement', loginMethod.toString());
  }

  signup() {
    this.eventEmitter('signup', 'engagement', 'signup');
  }

  viewItem(pet: Pet) {
    this.eventEmitter('view_item', 'engagement', pet.id + ' - ' + pet.name);
  }

  petAction(action: Action) {
    this.eventEmitter('pet_action', 'engagement', action.type);
  }

  private eventEmitter(
    eventName: string,
    eventCategory: string,
    eventAction: string,
    eventLabel: string = null,
    eventValue: number = null
  ) {
    gtag('event', eventName, {
      eventCategory: eventCategory,
      eventLabel: eventLabel,
      eventAction: eventAction,
      eventValue: eventValue,
    });
  }
}
