import { Injectable } from '@angular/core';
import { from, ReplaySubject } from 'rxjs';
import { take } from 'rxjs/operators';
import { Location } from '../models/Location';
import { NotificationService } from './notification.service';

import * as _ from 'lodash';
import { TranslateService } from '@ngx-translate/core';

@Injectable({
  providedIn: 'root'
})
export class GeolocationService {

  private geoLocationSbjct = new ReplaySubject<Location>(1);

  constructor(private notificationService: NotificationService, private translate: TranslateService) {
    setTimeout(() => {this.getLocation()}, 1000)
  }

  get location() {
    return this.geoLocationSbjct.asObservable();
  }

  private getLocation() {
    if (navigator.geolocation) {
      navigator.geolocation.getCurrentPosition(
        (position: Position) => {
        const location: Location = this.privatizeLocation(position);
        this.geoLocationSbjct.next(location)
      },
      () => {
        this.geoLocationSbjct.next(null);
        this.notificationService.bottomWarnMessage(this.translate.instant('response.bottom.not-activated'),  this.translate.instant('response.bottom.gelocation-title'));
      });
    } else {
      this.notificationService.bottomWarnMessage(this.translate.instant('response.bottom.not-compatible'),  this.translate.instant('response.bottom.gelocation-title'));
    }
  }

  private privatizeLocation(position: Position): Location {
    let lat, long;
    /**
     * !Privatize Geolocation by altering coordinates
     */
    // if (position.coords.accuracy < 200) {
    //   const latDecim = this.getDecimals(position.coords.latitude);
    //   const longDecim = this.getDecimals(position.coords.longitude);
    //   lat = position.coords.latitude + this.getRandom(latDecim);
    //   long = position.coords.longitude - this.getRandom(longDecim);
    // } else {
    //   lat = position.coords.latitude;
    //   long = position.coords.longitude;
    // }
    lat = position.coords.latitude;
    long = position.coords.longitude;
    return {lat, long}
  }

  private getRandom(num: number) {
    const res = _.random(1, 9) * (1 /( num * 10));
    return res;
  }

  private getDecimals(value: number) {
    if(Math.floor(value) === value) return 0;
    return value.toString().split(".")[1].length || 0;
  }


}
