import { Injectable } from '@angular/core';
import {
  HttpRequest,
  HttpHandler,
  HttpEvent,
  HttpInterceptor,
  HttpErrorResponse
} from '@angular/common/http';
import { Observable, throwError } from 'rxjs';
import { AuthService } from './auth.service';
import { catchError, map, switchMap, take, tap } from 'rxjs/operators';
import { AccessModalService } from '../content/access/access-modal.service';
import { Router } from '@angular/router';

@Injectable()
export class RestInterceptor implements HttpInterceptor {

  constructor(private authService: AuthService, private router: Router, private accessModalService: AccessModalService) {}

  intercept(request: HttpRequest<unknown>, next: HttpHandler): Observable<HttpEvent<unknown>> {
    const addTokenHeaderToReq = (token: String): void => {
      if (token) {
        request = request.clone({
          setHeaders: {
            Authorization: `Bearer ${token}`
          }
        });
      }
    };

    const manageErrorResponse = () => {
      return next.handle(request).pipe(
        catchError((error: HttpErrorResponse) => {
          if (error && error.status === 401) {
            this.router.navigate(['/']);
            if (!this.accessModalService.isActiveModal) {
              this.accessModalService.openAccessModal();
            }
          }
          return throwError(error);
        })
      );
    }
    return this.authService.token$.pipe(
      take(1),
      map(addTokenHeaderToReq),
      switchMap(manageErrorResponse)
    )
  }
}
