import { Injectable, Renderer2, RendererFactory2 } from '@angular/core';
import { Subject } from 'rxjs';

export interface ScrollEvent {
  scrollTop: number;
  scrollHeight: number;
  clientHeight: number;
  percent: number;
}

@Injectable({
  providedIn: 'root'
})
export class ScrollService {

  private scrollSbjct = new Subject<ScrollEvent>()

  constructor(rendererFactory: RendererFactory2) {
    const render = rendererFactory.createRenderer(null, null);
    render.listen(document.querySelector('#scrollindChildElementId').parentElement, 'scroll', (evt) => this.onScroll(evt))
  }

  private onScroll(event: Event) {
    const {scrollTop, scrollHeight, clientHeight} = event.target as any
    let percent = scrollTop / (scrollHeight - clientHeight ) * 100;
    percent = Math.round(percent * 100) / 100;
    this.scrollSbjct.next({scrollTop, scrollHeight, clientHeight, percent})
  }

  get scroll$() {
    return this.scrollSbjct.asObservable();
  }




}
