import { Injectable } from '@angular/core';
import { BehaviorSubject, Observable } from 'rxjs';
import { map, tap } from 'rxjs/operators';
import Page from '../models/Page';
import { PageParams } from '../models/PageParams';
import User from '../models/User';
import { RestService } from './rest.service';

@Injectable({
  providedIn: 'root'
})
export class UserService {

  private userSbjct = new BehaviorSubject<User>(JSON.parse(localStorage.getItem('user')) || null);

  constructor(private rest: RestService) { }

  setUser(user: User) {
    localStorage.setItem('user', JSON.stringify(user));
    this.userSbjct.next(user);
  }

  clearUser() {
    localStorage.removeItem('user')
    this.userSbjct.next(null);
  }

  get user(): Observable<User> {
    return this.userSbjct.asObservable();
  }

  get isAuthenticated(): Observable<boolean> {
    return this.userSbjct.pipe(
      map(data => data ? true : false)
    );
  }

  get isAdmin(): Observable<any> {
    return this.userSbjct.pipe(
      map((data: User) => data?.roles?.some(r => r.name === 'ROLE_ADMIN') ? true : false)
    );
  }

  fetchUser(email: string): Observable<User> {
    return this.rest.get(`/user?email=${email}`);
  }

  updateUser(user: User): Observable<User> {
    return this.rest.put(`/user?email=${user.email}`, user).pipe(
      tap(user => this.setUser(user))
    );
  }

  searchUsers(search = '', page: PageParams = { page: 0}): Observable<Page<User>> {
    return this.rest.get(`/admin/users?search=${search}&page=${page.page}&size=5`);
  }

  deleteUser(user: User) {
    return this.rest.delete(`/admin/user/${user.id}`);
  }
}
